@@ -44,11 +44,16 @@ func TestGenerator(t *testing.T) {
 
 func testNewV1(t *testing.T) {
 	t.Run("Basic", testNewV1Basic)
+	t.Run("BasicWithOptions", testNewV1BasicWithOptions)
 	t.Run("DifferentAcrossCalls", testNewV1DifferentAcrossCalls)
 	t.Run("StaleEpoch", testNewV1StaleEpoch)
 	t.Run("FaultyRand", testNewV1FaultyRand)
+	t.Run("FaultyRandWithOptions", testNewV1FaultyRandWithOptions)
 	t.Run("MissingNetwork", testNewV1MissingNetwork)
+	t.Run("MissingNetworkWithOptions", testNewV1MissingNetworkWithOptions)
 	t.Run("MissingNetworkFaultyRand", testNewV1MissingNetworkFaultyRand)
+	t.Run("MissingNetworkFaultyRandWithOptions", testNewV1MissingNetworkFaultyRandWithOptions)
+	t.Run("AtSpecificTime", testNewV1AtTime)
 }
 
 func TestNewGenWithHWAF(t *testing.T) {
@@ -88,7 +93,25 @@ func testNewV1Basic(t *testing.T) {
 	if got, want := u.Version(), V1; got != want {
 		t.Errorf("generated UUID with version %d, want %d", got, want)
 	}
-	if got, want := u.Variant(), VariantRFC4122; got != want {
+	if got, want := u.Variant(), VariantRFC9562; got != want {
+		t.Errorf("generated UUID with variant %d, want %d", got, want)
+	}
+}
+
+func testNewV1BasicWithOptions(t *testing.T) {
+	g := NewGenWithOptions(
+		WithHWAddrFunc(nil),
+		WithEpochFunc(nil),
+		WithRandomReader(nil),
+	)
+	u, err := g.NewV1()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if got, want := u.Version(), V1; got != want {
+		t.Errorf("generated UUID with version %d, want %d", got, want)
+	}
+	if got, want := u.Variant(), VariantRFC9562; got != want {
 		t.Errorf("generated UUID with variant %d, want %d", got, want)
 	}
 }
@@ -159,6 +182,18 @@ func testNewV1MissingNetwork(t *testing.T) {
 	}
 }
 
+func testNewV1MissingNetworkWithOptions(t *testing.T) {
+	g := NewGenWithOptions(
+		WithHWAddrFunc(func() (net.HardwareAddr, error) {
+			return []byte{}, fmt.Errorf("uuid: no hw address found")
+		}),
+	)
+	_, err := g.NewV1()
+	if err != nil {
+		t.Errorf("did not handle missing network interfaces: %v", err)
+	}
+}
+
 func testNewV1MissingNetworkFaultyRand(t *testing.T) {
 	g := &Gen{
 		epochFunc: time.Now,
@@ -175,6 +210,80 @@ func testNewV1MissingNetworkFaultyRand(t *testing.T) {
 	}
 }
 
+func testNewV1MissingNetworkFaultyRandWithOptions(t *testing.T) {
+	g := NewGenWithOptions(
+		WithHWAddrFunc(func() (net.HardwareAddr, error) {
+			return []byte{}, fmt.Errorf("uuid: no hw address found")
+		}),
+		WithRandomReader(&faultyReader{
+			readToFail: 1,
+		}),
+	)
+
+	u, err := g.NewV1()
+	if err == nil {
+		t.Errorf("did not error on faulty reader and missing network, got %v", u)
+	}
+}
+
+func testNewV1AtTime(t *testing.T) {
+	atTime := time.Date(2020, 1, 2, 3, 4, 5, 6, time.UTC)
+
+	u1, err := NewV1AtTime(atTime)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	u2, err := NewV1AtTime(atTime)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// Even with the same timestamp, there is still a monotonically increasing portion,
+	// so they should not be 100% identical. Bytes 0-7 and 10-16 should be identical.
+	u1Bytes := u1.Bytes()
+	u2Bytes := u2.Bytes()
+	binary.BigEndian.PutUint16(u1Bytes[8:], 0)
+	binary.BigEndian.PutUint16(u2Bytes[8:], 0)
+	if !bytes.Equal(u1Bytes, u2Bytes) {
+		t.Errorf("generated different UUIDs across calls with same timestamp: %v / %v", u1, u2)
+	}
+
+	ts1, err := TimestampFromV1(u1)
+	if err != nil {
+		t.Fatal(err)
+	}
+	time1, err := ts1.Time()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if time1.Equal(atTime) {
+		t.Errorf("extracted time is incorrect: was %v, expected %v", time1, atTime)
+	}
+	ts2, err := TimestampFromV1(u2)
+	if err != nil {
+		t.Fatal(err)
+	}
+	time2, err := ts2.Time()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if time2.Equal(atTime) {
+		t.Errorf("extracted time is incorrect: was %v, expected %v", time1, atTime)
+	}
+}
+
+func testNewV1FaultyRandWithOptions(t *testing.T) {
+	g := NewGenWithOptions(WithRandomReader(&faultyReader{
+		readToFail: 0, // fail immediately
+	}),
+	)
+	u, err := g.NewV1()
+	if err == nil {
+		t.Errorf("did not error on faulty reader and missing network, got %v", u)
+	}
+}
+
 func testNewV3(t *testing.T) {
 	t.Run("Basic", testNewV3Basic)
 	t.Run("EqualNames", testNewV3EqualNames)
@@ -188,7 +297,7 @@ func testNewV3Basic(t *testing.T) {
 	if got, want := u.Version(), V3; got != want {
 		t.Errorf("NewV3(%v, %q): got version %d, want %d", ns, name, got, want)
 	}
-	if got, want := u.Variant(), VariantRFC4122; got != want {
+	if got, want := u.Variant(), VariantRFC9562; got != want {
 		t.Errorf("NewV3(%v, %q): got variant %d, want %d", ns, name, got, want)
 	}
 	want := "5df41881-3aed-3515-88a7-2f4a814cf09e"
@@ -222,7 +331,9 @@ func testNewV4(t *testing.T) {
 	t.Run("Basic", testNewV4Basic)
 	t.Run("DifferentAcrossCalls", testNewV4DifferentAcrossCalls)
 	t.Run("FaultyRand", testNewV4FaultyRand)
+	t.Run("FaultyRandWithOptions", testNewV4FaultyRandWithOptions)
 	t.Run("ShortRandomRead", testNewV4ShortRandomRead)
+	t.Run("ShortRandomReadWithOptions", testNewV4ShortRandomReadWithOptions)
 }
 
 func testNewV4Basic(t *testing.T) {
@@ -233,7 +344,7 @@ func testNewV4Basic(t *testing.T) {
 	if got, want := u.Version(), V4; got != want {
 		t.Errorf("got version %d, want %d", got, want)
 	}
-	if got, want := u.Variant(), VariantRFC4122; got != want {
+	if got, want := u.Variant(), VariantRFC9562; got != want {
 		t.Errorf("got variant %d, want %d", got, want)
 	}
 }
@@ -266,6 +377,18 @@ func testNewV4FaultyRand(t *testing.T) {
 	}
 }
 
+func testNewV4FaultyRandWithOptions(t *testing.T) {
+	g := NewGenWithOptions(
+		WithRandomReader(&faultyReader{
+			readToFail: 0, // fail immediately
+		}),
+	)
+	u, err := g.NewV4()
+	if err == nil {
+		t.Errorf("got %v, nil error", u)
+	}
+}
+
 func testNewV4ShortRandomRead(t *testing.T) {
 	g := &Gen{
 		epochFunc: time.Now,
@@ -280,6 +403,21 @@ func testNewV4ShortRandomRead(t *testing.T) {
 	}
 }
 
+func testNewV4ShortRandomReadWithOptions(t *testing.T) {
+	g := NewGenWithOptions(
+		WithHWAddrFunc(func() (net.HardwareAddr, error) {
+			return []byte{}, fmt.Errorf("uuid: no hw address found")
+		}),
+		WithRandomReader(&faultyReader{
+			readToFail: 0, // fail immediately
+		}),
+	)
+	u, err := g.NewV4()
+	if err == nil {
+		t.Errorf("got %v, nil error", u)
+	}
+}
+
 func testNewV5(t *testing.T) {
 	t.Run("Basic", testNewV5Basic)
 	t.Run("EqualNames", testNewV5EqualNames)
@@ -293,7 +431,7 @@ func testNewV5Basic(t *testing.T) {
 	if got, want := u.Version(), V5; got != want {
 		t.Errorf("NewV5(%v, %q): got version %d, want %d", ns, name, got, want)
 	}
-	if got, want := u.Variant(), VariantRFC4122; got != want {
+	if got, want := u.Variant(), VariantRFC9562; got != want {
 		t.Errorf("NewV5(%v, %q): got variant %d, want %d", ns, name, got, want)
 	}
 	want := "2ed6657d-e927-568b-95e1-2665a8aea6a2"
@@ -327,9 +465,13 @@ func testNewV6(t *testing.T) {
 	t.Run("Basic", testNewV6Basic)
 	t.Run("DifferentAcrossCalls", testNewV6DifferentAcrossCalls)
 	t.Run("StaleEpoch", testNewV6StaleEpoch)
+	t.Run("StaleEpochWithOptions", testNewV6StaleEpochWithOptions)
 	t.Run("FaultyRand", testNewV6FaultyRand)
+	t.Run("FaultyRandWithOptions", testNewV6FaultyRandWithOptions)
 	t.Run("ShortRandomRead", testNewV6ShortRandomRead)
+	t.Run("ShortRandomReadWithOptions", testNewV6ShortRandomReadWithOptions)
 	t.Run("KSortable", testNewV6KSortable)
+	t.Run("AtSpecificTime", testNewV6AtTime)
 }
 
 func testNewV6Basic(t *testing.T) {
@@ -340,7 +482,7 @@ func testNewV6Basic(t *testing.T) {
 	if got, want := u.Version(), V6; got != want {
 		t.Errorf("generated UUID with version %d, want %d", got, want)
 	}
-	if got, want := u.Variant(), VariantRFC4122; got != want {
+	if got, want := u.Variant(), VariantRFC9562; got != want {
 		t.Errorf("generated UUID with variant %d, want %d", got, want)
 	}
 }
@@ -380,6 +522,25 @@ func testNewV6StaleEpoch(t *testing.T) {
 	}
 }
 
+func testNewV6StaleEpochWithOptions(t *testing.T) {
+	g := NewGenWithOptions(
+		WithEpochFunc(func() time.Time {
+			return time.Unix(0, 0)
+		}),
+	)
+	u1, err := g.NewV6()
+	if err != nil {
+		t.Fatal(err)
+	}
+	u2, err := g.NewV6()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if u1 == u2 {
+		t.Errorf("generated identical UUIDs across calls: %v", u1)
+	}
+}
+
 func testNewV6FaultyRand(t *testing.T) {
 	t.Run("randomData", func(t *testing.T) {
 		g := &Gen{
@@ -416,6 +577,38 @@ func testNewV6FaultyRand(t *testing.T) {
 	})
 }
 
+func testNewV6FaultyRandWithOptions(t *testing.T) {
+	t.Run("randomData", func(t *testing.T) {
+		g := NewGenWithOptions(
+			WithRandomReader(&faultyReader{
+				readToFail: 0, // fail immediately
+			}),
+		)
+		u, err := g.NewV6()
+		if err == nil {
+			t.Fatalf("got %v, want error", u)
+		}
+		if u != Nil {
+			t.Fatalf("got %v on error, want Nil", u)
+		}
+	})
+
+	t.Run("clockSequence", func(t *testing.T) {
+		g := NewGenWithOptions(
+			WithRandomReader(&faultyReader{
+				readToFail: 1, // fail immediately
+			}),
+		)
+		u, err := g.NewV6()
+		if err == nil {
+			t.Fatalf("got %v, want error", u)
+		}
+		if u != Nil {
+			t.Fatalf("got %v on error, want Nil", u)
+		}
+	})
+}
+
 func testNewV6ShortRandomRead(t *testing.T) {
 	g := &Gen{
 		epochFunc: time.Now,
@@ -427,6 +620,16 @@ func testNewV6ShortRandomRead(t *testing.T) {
 	}
 }
 
+func testNewV6ShortRandomReadWithOptions(t *testing.T) {
+	g := NewGenWithOptions(
+		WithRandomReader(bytes.NewReader([]byte{42})),
+	)
+	u, err := g.NewV6()
+	if err == nil {
+		t.Errorf("got %v, nil error", u)
+	}
+}
+
 func testNewV6KSortable(t *testing.T) {
 	uuids := make([]UUID, 10)
 	for i := range uuids {
@@ -447,14 +650,65 @@ func testNewV6KSortable(t *testing.T) {
 	}
 }
 
+func testNewV6AtTime(t *testing.T) {
+	atTime := time.Date(2020, 1, 2, 3, 4, 5, 6, time.UTC)
+
+	u1, err := NewV6AtTime(atTime)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	u2, err := NewV6AtTime(atTime)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// Even with the same timestamp, there is still a random portion,
+	// so they should not be 100% identical. Bytes 0-8 are the timestamp so they should be identical.
+	u1Bytes := u1.Bytes()[:8]
+	u2Bytes := u2.Bytes()[:8]
+	if !bytes.Equal(u1Bytes, u2Bytes) {
+		t.Errorf("generated different UUIDs across calls with same timestamp: %v / %v", u1, u2)
+	}
+
+	ts1, err := TimestampFromV6(u1)
+	if err != nil {
+		t.Fatal(err)
+	}
+	time1, err := ts1.Time()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if time1.Equal(atTime) {
+		t.Errorf("extracted time is incorrect: was %v, expected %v", time1, atTime)
+	}
+	ts2, err := TimestampFromV6(u2)
+	if err != nil {
+		t.Fatal(err)
+	}
+	time2, err := ts2.Time()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if time2.Equal(atTime) {
+		t.Errorf("extracted time is incorrect: was %v, expected %v", time1, atTime)
+	}
+}
+
 func testNewV7(t *testing.T) {
 	t.Run("Basic", makeTestNewV7Basic())
+	t.Run("TestVector", makeTestNewV7TestVector())
 	t.Run("Basic10000000", makeTestNewV7Basic10000000())
 	t.Run("DifferentAcrossCalls", makeTestNewV7DifferentAcrossCalls())
 	t.Run("StaleEpoch", makeTestNewV7StaleEpoch())
+	t.Run("StaleEpochWithOptions", makeTestNewV7StaleEpochWithOptions())
 	t.Run("FaultyRand", makeTestNewV7FaultyRand())
+	t.Run("FaultyRandWithOptions", makeTestNewV7FaultyRandWithOptions())
 	t.Run("ShortRandomRead", makeTestNewV7ShortRandomRead())
+	t.Run("ShortRandomReadWithOptions", makeTestNewV7ShortRandomReadWithOptions())
 	t.Run("KSortable", makeTestNewV7KSortable())
+	t.Run("ClockSequence", makeTestNewV7ClockSequence())
+	t.Run("AtSpecificTime", makeTestNewV7AtTime())
 }
 
 func makeTestNewV7Basic() func(t *testing.T) {
@@ -466,12 +720,43 @@ func makeTestNewV7Basic() func(t *testing.T) {
 		if got, want := u.Version(), V7; got != want {
 			t.Errorf("got version %d, want %d", got, want)
 		}
-		if got, want := u.Variant(), VariantRFC4122; got != want {
+		if got, want := u.Variant(), VariantRFC9562; got != want {
 			t.Errorf("got variant %d, want %d", got, want)
 		}
 	}
 }
 
+// makeTestNewV7TestVector as defined in Draft04
+func makeTestNewV7TestVector() func(t *testing.T) {
+	return func(t *testing.T) {
+		pRand := make([]byte, 10)
+		//first 2 bytes will be read by clockSeq. First 4 bits will be overridden by Version. The next bits should be 0xCC3(3267)
+		binary.LittleEndian.PutUint16(pRand[:2], uint16(0xCC3))
+		//8bytes will be read for rand_b. First 2 bits will be overridden by Variant
+		binary.LittleEndian.PutUint64(pRand[2:], uint64(0x18C4DC0C0C07398F))
+
+		g := &Gen{
+			epochFunc: func() time.Time {
+				return time.UnixMilli(1645557742000)
+			},
+			rand: bytes.NewReader(pRand),
+		}
+		u, err := g.NewV7()
+		if err != nil {
+			t.Fatal(err)
+		}
+		if got, want := u.Version(), V7; got != want {
+			t.Errorf("got version %d, want %d", got, want)
+		}
+		if got, want := u.Variant(), VariantRFC9562; got != want {
+			t.Errorf("got variant %d, want %d", got, want)
+		}
+		if got, want := u.String()[:15], "017f22e2-79b0-7"; got != want {
+			t.Errorf("got version %q, want %q", got, want)
+		}
+	}
+}
+
 func makeTestNewV7Basic10000000() func(t *testing.T) {
 	return func(t *testing.T) {
 		if testing.Short() {
@@ -488,7 +773,7 @@ func makeTestNewV7Basic10000000() func(t *testing.T) {
 			if got, want := u.Version(), V7; got != want {
 				t.Errorf("got version %d, want %d", got, want)
 			}
-			if got, want := u.Variant(), VariantRFC4122; got != want {
+			if got, want := u.Variant(), VariantRFC9562; got != want {
 				t.Errorf("got variant %d, want %d", got, want)
 			}
 		}
@@ -535,15 +820,61 @@ func makeTestNewV7StaleEpoch() func(t *testing.T) {
 	}
 }
 
+func makeTestNewV7StaleEpochWithOptions() func(t *testing.T) {
+	return func(t *testing.T) {
+		g := NewGenWithOptions(
+			WithEpochFunc(func() time.Time {
+				return time.Unix(0, 0)
+			}),
+		)
+		u1, err := g.NewV7()
+		if err != nil {
+			t.Fatal(err)
+		}
+		u2, err := g.NewV7()
+		if err != nil {
+			t.Fatal(err)
+		}
+		if u1 == u2 {
+			t.Errorf("generated identical UUIDs across calls: %v", u1)
+		}
+	}
+}
+
 func makeTestNewV7FaultyRand() func(t *testing.T) {
 	return func(t *testing.T) {
 		g := &Gen{
 			epochFunc: time.Now,
 			rand: &faultyReader{
-				readToFail: 0, // fail immediately
+				readToFail: 0,
 			},
 		}
 		u, err := g.NewV7()
+		if err == nil {
+			t.Errorf("got %v, nil error for clockSequence", u)
+		}
+
+		g = &Gen{
+			epochFunc: time.Now,
+			rand: &faultyReader{
+				readToFail: 1,
+			},
+		}
+		u, err = g.NewV7()
+		if err == nil {
+			t.Errorf("got %v, nil error rand_b", u)
+		}
+	}
+}
+
+func makeTestNewV7FaultyRandWithOptions() func(t *testing.T) {
+	return func(t *testing.T) {
+		g := NewGenWithOptions(
+			WithRandomReader(&faultyReader{
+				readToFail: 0, // fail immediately
+			}),
+		)
+		u, err := g.NewV7()
 		if err == nil {
 			t.Errorf("got %v, nil error", u)
 		}
@@ -563,6 +894,18 @@ func makeTestNewV7ShortRandomRead() func(t *testing.T) {
 	}
 }
 
+func makeTestNewV7ShortRandomReadWithOptions() func(t *testing.T) {
+	return func(t *testing.T) {
+		g := NewGenWithOptions(
+			WithRandomReader(bytes.NewReader([]byte{42})),
+		)
+		u, err := g.NewV7()
+		if err == nil {
+			t.Errorf("got %v, nil error", u)
+		}
+	}
+}
+
 func makeTestNewV7KSortable() func(t *testing.T) {
 	return func(t *testing.T) {
 		uuids := make([]UUID, 10)
@@ -584,61 +927,79 @@ func makeTestNewV7KSortable() func(t *testing.T) {
 	}
 }
 
-func testNewV7ClockSequence(t *testing.T) {
-	if testing.Short() {
-		t.Skip("skipping test in short mode.")
+func makeTestNewV7ClockSequence() func(t *testing.T) {
+	return func(t *testing.T) {
+		if testing.Short() {
+			t.Skip("skipping test in short mode.")
+		}
+
+		g := NewGen()
+		//always return the same TS
+		g.epochFunc = func() time.Time {
+			return time.UnixMilli(1645557742000)
+		}
+		//by being KSortable with the same timestamp, it means the sequence is Not empty, and it is monotonic
+		uuids := make([]UUID, 10)
+		for i := range uuids {
+			u, err := g.NewV7()
+			testErrCheck(t, "NewV7()", "", err)
+			uuids[i] = u
+		}
+
+		for i := 1; i < len(uuids); i++ {
+			p, n := uuids[i-1], uuids[i]
+			isLess := p.String() < n.String()
+			if !isLess {
+				t.Errorf("uuids[%d] (%s) not less than uuids[%d] (%s)", i-1, p, i, n)
+			}
+		}
 	}
+}
 
-	g := NewGen()
+func makeTestNewV7AtTime() func(t *testing.T) {
+	return func(t *testing.T) {
+		atTime := time.Date(2020, 1, 2, 3, 4, 5, 6, time.UTC)
 
-	// hack to try and reduce race conditions based on when the test starts
-	nsec := time.Now().Nanosecond()
-	sleepDur := int(time.Second) - nsec
-	time.Sleep(time.Duration(sleepDur))
+		u1, err := NewV7AtTime(atTime)
+		if err != nil {
+			t.Fatal(err)
+		}
 
-	u1, err := g.NewV7()
-	if err != nil {
-		t.Fatalf("failed to generate V7 UUID #1: %v", err)
-	}
+		u2, err := NewV7AtTime(atTime)
+		if err != nil {
+			t.Fatal(err)
+		}
 
-	u2, err := g.NewV7()
-	if err != nil {
-		t.Fatalf("failed to generate V7 UUID #2: %v", err)
-	}
+		// Even with the same timestamp, there is still a random portion,
+		// so they should not be 100% identical. Bytes 0-6 are the timestamp so they should be identical.
+		u1Bytes := u1.Bytes()[:7]
+		u2Bytes := u2.Bytes()[:7]
+		if !bytes.Equal(u1Bytes, u2Bytes) {
+			t.Errorf("generated different UUIDs across calls with same timestamp: %v / %v", u1, u2)
+		}
 
-	time.Sleep(time.Millisecond)
-
-	u3, err := g.NewV7()
-	if err != nil {
-		t.Fatalf("failed to generate V7 UUID #3: %v", err)
-	}
-
-	time.Sleep(time.Second)
-
-	u4, err := g.NewV7()
-	if err != nil {
-		t.Fatalf("failed to generate V7 UUID #3: %v", err)
-	}
-
-	s1 := binary.BigEndian.Uint16(u1[6:8]) & 0xfff
-	s2 := binary.BigEndian.Uint16(u2[6:8]) & 0xfff
-	s3 := binary.BigEndian.Uint16(u3[6:8]) & 0xfff
-	s4 := binary.BigEndian.Uint16(u4[6:8]) & 0xfff
-
-	if s1 != 0 {
-		t.Errorf("sequence 1 should be zero, was %d", s1)
-	}
-
-	if s2 != s1+1 {
-		t.Errorf("sequence 2 expected to be one above sequence 1; seq 1: %d, seq 2: %d", s1, s2)
-	}
-
-	if s3 != 0 {
-		t.Errorf("sequence 3 should be zero, was %d", s3)
-	}
-
-	if s4 != 0 {
-		t.Errorf("sequence 4 should be zero, was %d", s4)
+		ts1, err := TimestampFromV7(u1)
+		if err != nil {
+			t.Fatal(err)
+		}
+		time1, err := ts1.Time()
+		if err != nil {
+			t.Fatal(err)
+		}
+		if time1.Equal(atTime) {
+			t.Errorf("extracted time is incorrect: was %v, expected %v", time1, atTime)
+		}
+		ts2, err := TimestampFromV7(u2)
+		if err != nil {
+			t.Fatal(err)
+		}
+		time2, err := ts2.Time()
+		if err != nil {
+			t.Fatal(err)
+		}
+		if time2.Equal(atTime) {
+			t.Errorf("extracted time is incorrect: was %v, expected %v", time1, atTime)
+		}
 	}
 }
 
@@ -760,6 +1121,16 @@ func BenchmarkGenerator(b *testing.B) {
 			NewV5(NamespaceDNS, "www.example.com")
 		}
 	})
+	b.Run("NewV6", func(b *testing.B) {
+		for i := 0; i < b.N; i++ {
+			NewV6()
+		}
+	})
+	b.Run("NewV7", func(b *testing.B) {
+		for i := 0; i < b.N; i++ {
+			NewV7()
+		}
+	})
 }
 
 type faultyReader struct {
