@@ -20,24 +20,21 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 // Package uuid provides implementations of the Universally Unique Identifier
-// (UUID), as specified in RFC-4122 and the Peabody RFC Draft (revision 03).
+// (UUID), as specified in RFC-9562 (formerly RFC-4122).
 //
-// RFC-4122[1] provides the specification for versions 1, 3, 4, and 5. The
-// Peabody UUID RFC Draft[2] provides the specification for the new k-sortable
-// UUIDs, versions 6 and 7.
+// RFC-9562[1] provides the specification for versions 1, 3, 4, 5, 6 and 7.
 //
-// DCE 1.1[3] provides the specification for version 2, but version 2 support
+// DCE 1.1[2] provides the specification for version 2, but version 2 support
 // was removed from this package in v4 due to some concerns with the
 // specification itself. Reading the spec, it seems that it would result in
 // generating UUIDs that aren't very unique. In having read the spec it seemed
 // that our implementation did not meet the spec. It also seems to be at-odds
-// with RFC 4122, meaning we would need quite a bit of special code to support
+// with RFC 9562, meaning we would need quite a bit of special code to support
 // it. Lastly, there were no Version 2 implementations that we could find to
 // ensure we were understanding the specification correctly.
 //
-// [1] https://tools.ietf.org/html/rfc4122
-// [2] https://datatracker.ietf.org/doc/html/draft-peabody-dispatch-new-uuid-format-03
-// [3] http://pubs.opengroup.org/onlinepubs/9696989899/chap5.htm#tagcjh_08_02_01_01
+// [1] https://tools.ietf.org/html/rfc9562
+// [2] http://pubs.opengroup.org/onlinepubs/9696989899/chap5.htm#tagcjh_08_02_01_01
 package uuid
 
 import (
@@ -50,7 +47,7 @@ import (
 // Size of a UUID in bytes.
 const Size = 16
 
-// UUID is an array type to represent the value of a UUID, as defined in RFC-4122.
+// UUID is an array type to represent the value of a UUID, as defined in RFC-9562.
 type UUID [Size]byte
 
 // UUID versions.
@@ -61,19 +58,22 @@ const (
 	V3      // Version 3 (namespace name-based)
 	V4      // Version 4 (random)
 	V5      // Version 5 (namespace name-based)
-	V6      // Version 6 (k-sortable timestamp and random data, field-compatible with v1) [peabody draft]
-	V7      // Version 7 (k-sortable timestamp and random data) [peabody draft]
-	_       // Version 8 (k-sortable timestamp, meant for custom implementations) [peabody draft] [not implemented]
+	V6      // Version 6 (k-sortable timestamp and random data, field-compatible with v1)
+	V7      // Version 7 (k-sortable timestamp and random data)
+	_       // Version 8 (k-sortable timestamp, meant for custom implementations) [not implemented]
 )
 
 // UUID layout variants.
 const (
 	VariantNCS byte = iota
-	VariantRFC4122
+	VariantRFC9562
 	VariantMicrosoft
 	VariantFuture
 )
 
+// Backward-compatible variant for RFC 4122
+const VariantRFC4122 = VariantRFC9562
+
 // UUID DCE domains.
 const (
 	DomainPerson = iota
@@ -82,13 +82,19 @@ const (
 )
 
 // Timestamp is the count of 100-nanosecond intervals since 00:00:00.00,
-// 15 October 1582 within a V1 UUID. This type has no meaning for other
-// UUID versions since they don't have an embedded timestamp.
+// 15 October 1582 within a V1 or V6 UUID, or as a common intermediate
+// representation of the (Unix Millisecond) timestamp within a V7 UUID.
+// This type has no meaning for other UUID versions since they don't
+// have an embedded timestamp.
 type Timestamp uint64
 
 const _100nsPerSecond = 10000000
 
-// Time returns the UTC time.Time representation of a Timestamp
+// Time returns the time.Time representation of a Timestamp.
+//
+// The resulting time.Time:
+// - Contains only wall clock time (no monotonic clock component)
+// - Uses the local system timezone for the location
 func (t Timestamp) Time() (time.Time, error) {
 	secs := uint64(t) / _100nsPerSecond
 	nsecs := 100 * (uint64(t) % _100nsPerSecond)
@@ -100,7 +106,7 @@ func (t Timestamp) Time() (time.Time, error) {
 // Returns an error if the UUID is any version other than 1.
 func TimestampFromV1(u UUID) (Timestamp, error) {
 	if u.Version() != 1 {
-		err := fmt.Errorf("uuid: %s is version %d, not version 1", u, u.Version())
+		err := fmt.Errorf("%w %s is version %d, not version 1", ErrInvalidVersion, u, u.Version())
 		return 0, err
 	}
 
@@ -113,15 +119,9 @@ func TimestampFromV1(u UUID) (Timestamp, error) {
 
 // TimestampFromV6 returns the Timestamp embedded within a V6 UUID. This
 // function returns an error if the UUID is any version other than 6.
-//
-// This is implemented based on revision 03 of the Peabody UUID draft, and may
-// be subject to change pending further revisions. Until the final specification
-// revision is finished, changes required to implement updates to the spec will
-// not be considered a breaking change. They will happen as a minor version
-// releases until the spec is final.
 func TimestampFromV6(u UUID) (Timestamp, error) {
 	if u.Version() != 6 {
-		return 0, fmt.Errorf("uuid: %s is version %d, not version 6", u, u.Version())
+		return 0, fmt.Errorf("%w %s is version %d, not version 6", ErrInvalidVersion, u, u.Version())
 	}
 
 	hi := binary.BigEndian.Uint32(u[0:4])
@@ -133,15 +133,9 @@ func TimestampFromV6(u UUID) (Timestamp, error) {
 
 // TimestampFromV7 returns the Timestamp embedded within a V7 UUID. This
 // function returns an error if the UUID is any version other than 7.
-//
-// This is implemented based on revision 03 of the Peabody UUID draft, and may
-// be subject to change pending further revisions. Until the final specification
-// revision is finished, changes required to implement updates to the spec will
-// not be considered a breaking change. They will happen as a minor version
-// releases until the spec is final.
 func TimestampFromV7(u UUID) (Timestamp, error) {
 	if u.Version() != 7 {
-		return 0, fmt.Errorf("uuid: %s is version %d, not version 7", u, u.Version())
+		return 0, fmt.Errorf("%w %s is version %d, not version 7", ErrInvalidVersion, u, u.Version())
 	}
 
 	t := 0 |
@@ -152,17 +146,20 @@ func TimestampFromV7(u UUID) (Timestamp, error) {
 		(int64(u[4]) << 8) |
 		int64(u[5])
 
-	// convert to format expected by Timestamp
-	tsNanos := epochStart + time.UnixMilli(t).UTC().UnixNano()/100
+	// UUIDv7 stores MS since 1970-01-01 00:00:00, but the Timestamp
+	// type stores 100-nanosecond increments since 1582-10-15 00:00:00.
+	// This conversion multiplies ms by 10,000 to get 100-ns chunks and adds
+	// the difference between October 1582 and January 1970.
+	tsNanos := epochStart + (t * 10000)
 	return Timestamp(tsNanos), nil
 }
 
-// Nil is the nil UUID, as specified in RFC-4122, that has all 128 bits set to
+// Nil is the nil UUID, as specified in RFC-9562, that has all 128 bits set to
 // zero.
 var Nil = UUID{}
 
-// Max is the maximum UUID, as specified in a draft of RFC-4122, that has all 128 bits
-// set to one. This feature is subject to removal if it is removed from the final draft of the RFC.
+// Max is the maximum UUID, as specified in RFC-9562, that has all 128 bits
+// set to one.
 var Max = UUID{
 	0xFF,
 	0xFF,
@@ -206,7 +203,7 @@ func (u UUID) Variant() byte {
 	case (u[8] >> 7) == 0x00:
 		return VariantNCS
 	case (u[8] >> 6) == 0x02:
-		return VariantRFC4122
+		return VariantRFC9562
 	case (u[8] >> 5) == 0x06:
 		return VariantMicrosoft
 	case (u[8] >> 5) == 0x07:
@@ -221,7 +218,7 @@ func (u UUID) Bytes() []byte {
 	return u[:]
 }
 
-// encodeCanonical encodes the canonical RFC-4122 form of UUID u into the
+// encodeCanonical encodes the canonical RFC-9562 form of UUID u into the
 // first 36 bytes dst.
 func encodeCanonical(dst []byte, u UUID) {
 	const hextable = "0123456789abcdef"
@@ -242,7 +239,7 @@ func encodeCanonical(dst []byte, u UUID) {
 	}
 }
 
-// String returns a canonical RFC-4122 string representation of the UUID:
+// String returns a canonical RFC-9562 string representation of the UUID:
 // xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx.
 func (u UUID) String() string {
 	var buf [36]byte
@@ -254,8 +251,8 @@ func (u UUID) String() string {
 //
 // The behavior is as follows:
 // The 'x' and 'X' verbs output only the hex digits of the UUID, using a-f for 'x' and A-F for 'X'.
-// The 'v', '+v', 's' and 'q' verbs return the canonical RFC-4122 string representation.
-// The 'S' verb returns the RFC-4122 format, but with capital hex digits.
+// The 'v', '+v', 's' and 'q' verbs return the canonical RFC-9562 string representation.
+// The 'S' verb returns the RFC-9562 format, but with capital hex digits.
 // The '#v' verb returns the "Go syntax" representation, which is a 16 byte array initializer.
 // All other verbs not handled directly by the fmt package (like '%p') are unsupported and will return
 // "%!verb(uuid.UUID=value)" as recommended by the fmt package.
@@ -308,7 +305,7 @@ func (u *UUID) SetVariant(v byte) {
 	switch v {
 	case VariantNCS:
 		u[8] = (u[8]&(0xff>>1) | (0x00 << 7))
-	case VariantRFC4122:
+	case VariantRFC9562:
 		u[8] = (u[8]&(0xff>>2) | (0x02 << 6))
 	case VariantMicrosoft:
 		u[8] = (u[8]&(0xff>>3) | (0x06 << 5))
